# ANALYSIS Visualisation/"Volcano plot from existing results" (Plots a Volcano plot from results of a statistical
# test. A suitable output for this tool can be generated by running a two-group or a several-group statistical test
# or a linear model using empiricalBayes.)
# INPUT GENE_EXPRS normalized.tsv OUTPUT volcanoP.png
# PARAMETER expression.column COLUMN_SEL (Column that contains the expression values)
# PARAMETER p.value.column COLUMN_SEL (Column that contains the p-values)
# PARAMETER image.width INTEGER FROM 200 TO 3200 DEFAULT 600 (Width of the plotted network image)
# PARAMETER image.height INTEGER FROM 200 TO 3200 DEFAULT 600 (Height of the plotted network image)


# Volcano plot from existing results
# JTT 8.11.2007

# Renaming variables
w<-image.width
h<-image.height

# Loads the normalized data
file<-c("normalized.tsv")
dat<-read.table(file, header=T, sep="\t", row.names=1)

# Sanity checks
if(expression.column=="empty") {
   stop("You haven't selected a phenodata column for expression! Tool cannot be executed.")
}
if(p.value.column=="empty") {
   stop("You haven't selected a phenodata column for p-value! Tool cannot be executed.")
}
if(sum(grep("p.adjusted", colnames(dat)))==0) {
   stop("You don't have any P-value columns in the dataset! Please run some statistical test first.")
}

# Extracts the data
expression<-dat[,grep(expression.column, colnames(dat))]
pvalues<-dat[,grep(p.value.column, colnames(dat))]

# Plotting
bitmap(file="volcanoP.png", width=w/72, height=h/72)
plot(expression, -log(pvalues), xlim=c(-max(expression), max(expression)), main="Volcano plot", pch=19, xlab="Mean expression", ylab="-log(p)")
dev.off()
